function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

import { TokenType } from "./token";
import * as regex from "./regexFactory";
import TokenizerEngine from "./TokenizerEngine";
import { escapeRegExp } from "./regexUtil";
import { equalizeWhitespace } from "../utils";
import { NestedComment } from "./NestedComment";

var Tokenizer = /*#__PURE__*/function () {
  function Tokenizer(cfg) {
    _classCallCheck(this, Tokenizer);

    this.cfg = cfg;
    this.rulesBeforeParams = this.buildRulesBeforeParams(cfg);
    this.rulesAfterParams = this.buildRulesAfterParams(cfg);
  }

  _createClass(Tokenizer, [{
    key: "tokenize",
    value: function tokenize(input, paramTypesOverrides) {
      var rules = [].concat(_toConsumableArray(this.rulesBeforeParams), _toConsumableArray(this.buildParamRules(this.cfg, paramTypesOverrides)), _toConsumableArray(this.rulesAfterParams));
      var tokens = new TokenizerEngine(rules).tokenize(input);
      return this.cfg.postProcess ? this.cfg.postProcess(tokens) : tokens;
    } // These rules can be cached as they only depend on
    // the Tokenizer config options specified for each SQL dialect

  }, {
    key: "buildRulesBeforeParams",
    value: function buildRulesBeforeParams(cfg) {
      var _cfg$lineCommentTypes, _cfg$reservedPhrases;

      return this.validRules([{
        type: TokenType.BLOCK_COMMENT,
        regex: cfg.nestedBlockComments ? new NestedComment() : new RegExp("(\\/\\*(?:(?![])[\\s\\S])*?\\*\\/)", "y")
      }, {
        type: TokenType.LINE_COMMENT,
        regex: regex.lineComment((_cfg$lineCommentTypes = cfg.lineCommentTypes) !== null && _cfg$lineCommentTypes !== void 0 ? _cfg$lineCommentTypes : ['--'])
      }, {
        type: TokenType.QUOTED_IDENTIFIER,
        regex: regex.string(cfg.identTypes)
      }, {
        type: TokenType.NUMBER,
        regex: new RegExp("(?:0x[0-9A-Fa-f]+|0b[01]+|(?:\\x2D[\\t-\\r \\xA0\\u1680\\u2000-\\u200A\\u2028\\u2029\\u202F\\u205F\\u3000\\uFEFF]*)?[0-9]+(?:\\.[0-9]*)?(?:[Ee][\\+\\x2D]?[0-9]+(?:\\.[0-9]+)?)?)(?![0-9A-Z_a-z])", "y")
      }, // RESERVED_PHRASE is matched before all other keyword tokens
      // to e.g. prioritize matching "TIMESTAMP WITH TIME ZONE" phrase over "WITH" command.
      {
        type: TokenType.RESERVED_PHRASE,
        regex: regex.reservedWord((_cfg$reservedPhrases = cfg.reservedPhrases) !== null && _cfg$reservedPhrases !== void 0 ? _cfg$reservedPhrases : [], cfg.identChars),
        text: toCanonical
      }, {
        type: TokenType.CASE,
        regex: new RegExp("CA[S\\u017F]E\\b", "iy"),
        text: toCanonical
      }, {
        type: TokenType.END,
        regex: new RegExp("END\\b", "iy"),
        text: toCanonical
      }, {
        type: TokenType.BETWEEN,
        regex: new RegExp("BETWEEN\\b", "iy"),
        text: toCanonical
      }, {
        type: TokenType.LIMIT,
        regex: cfg.reservedCommands.includes('LIMIT') ? new RegExp("LIMIT\\b", "iy") : undefined,
        text: toCanonical
      }, {
        type: TokenType.RESERVED_COMMAND,
        regex: regex.reservedWord(cfg.reservedCommands, cfg.identChars),
        text: toCanonical
      }, {
        type: TokenType.RESERVED_SELECT,
        regex: regex.reservedWord(cfg.reservedSelect, cfg.identChars),
        text: toCanonical
      }, {
        type: TokenType.RESERVED_SET_OPERATION,
        regex: regex.reservedWord(cfg.reservedSetOperations, cfg.identChars),
        text: toCanonical
      }, {
        type: TokenType.WHEN,
        regex: new RegExp("WHEN\\b", "iy"),
        text: toCanonical
      }, {
        type: TokenType.ELSE,
        regex: new RegExp("EL[S\\u017F]E\\b", "iy"),
        text: toCanonical
      }, {
        type: TokenType.THEN,
        regex: new RegExp("THEN\\b", "iy"),
        text: toCanonical
      }, {
        type: TokenType.RESERVED_JOIN,
        regex: regex.reservedWord(cfg.reservedJoins, cfg.identChars),
        text: toCanonical
      }, {
        type: TokenType.AND,
        regex: new RegExp("AND\\b", "iy"),
        text: toCanonical
      }, {
        type: TokenType.OR,
        regex: new RegExp("OR\\b", "iy"),
        text: toCanonical
      }, {
        type: TokenType.XOR,
        regex: cfg.supportsXor ? new RegExp("XOR\\b", "iy") : undefined,
        text: toCanonical
      }, {
        type: TokenType.RESERVED_FUNCTION_NAME,
        regex: regex.reservedWord(cfg.reservedFunctionNames, cfg.identChars),
        text: toCanonical
      }, {
        type: TokenType.RESERVED_KEYWORD,
        regex: regex.reservedWord(cfg.reservedKeywords, cfg.identChars),
        text: toCanonical
      }]);
    } // These rules can also be cached as they only depend on
    // the Tokenizer config options specified for each SQL dialect

  }, {
    key: "buildRulesAfterParams",
    value: function buildRulesAfterParams(cfg) {
      var _cfg$operators;

      return this.validRules([{
        type: TokenType.VARIABLE,
        regex: cfg.variableTypes ? regex.variable(cfg.variableTypes) : undefined
      }, {
        type: TokenType.STRING,
        regex: regex.string(cfg.stringTypes)
      }, {
        type: TokenType.IDENTIFIER,
        regex: regex.identifier(cfg.identChars)
      }, {
        type: TokenType.DELIMITER,
        regex: new RegExp(";", "y")
      }, {
        type: TokenType.COMMA,
        regex: new RegExp("[,]", "y")
      }, {
        type: TokenType.OPEN_PAREN,
        regex: regex.parenthesis('open', cfg.extraParens)
      }, {
        type: TokenType.CLOSE_PAREN,
        regex: regex.parenthesis('close', cfg.extraParens)
      }, {
        type: TokenType.OPERATOR,
        regex: regex.operator([// standard operators
        '+', '-', '/', '>', '<', '=', '<>', '<=', '>=', '!='].concat(_toConsumableArray((_cfg$operators = cfg.operators) !== null && _cfg$operators !== void 0 ? _cfg$operators : [])))
      }, {
        type: TokenType.ASTERISK,
        regex: new RegExp("\\*", "y")
      }, {
        type: TokenType.DOT,
        regex: new RegExp("\\.", "y")
      }]);
    } // These rules can't be blindly cached as the paramTypesOverrides object
    // can differ on each invocation of the format() function.

  }, {
    key: "buildParamRules",
    value: function buildParamRules(cfg, paramTypesOverrides) {
      var _cfg$paramTypes, _cfg$paramTypes2, _cfg$paramTypes3, _cfg$paramTypes4;

      // Each dialect has its own default parameter types (if any),
      // but these can be overriden by the user of the library.
      var paramTypes = {
        named: (paramTypesOverrides === null || paramTypesOverrides === void 0 ? void 0 : paramTypesOverrides.named) || ((_cfg$paramTypes = cfg.paramTypes) === null || _cfg$paramTypes === void 0 ? void 0 : _cfg$paramTypes.named) || [],
        quoted: (paramTypesOverrides === null || paramTypesOverrides === void 0 ? void 0 : paramTypesOverrides.quoted) || ((_cfg$paramTypes2 = cfg.paramTypes) === null || _cfg$paramTypes2 === void 0 ? void 0 : _cfg$paramTypes2.quoted) || [],
        numbered: (paramTypesOverrides === null || paramTypesOverrides === void 0 ? void 0 : paramTypesOverrides.numbered) || ((_cfg$paramTypes3 = cfg.paramTypes) === null || _cfg$paramTypes3 === void 0 ? void 0 : _cfg$paramTypes3.numbered) || [],
        positional: typeof (paramTypesOverrides === null || paramTypesOverrides === void 0 ? void 0 : paramTypesOverrides.positional) === 'boolean' ? paramTypesOverrides.positional : (_cfg$paramTypes4 = cfg.paramTypes) === null || _cfg$paramTypes4 === void 0 ? void 0 : _cfg$paramTypes4.positional
      };
      return this.validRules([{
        type: TokenType.NAMED_PARAMETER,
        regex: regex.parameter(paramTypes.named, regex.identifierPattern(cfg.paramChars || cfg.identChars)),
        key: function key(v) {
          return v.slice(1);
        }
      }, {
        type: TokenType.QUOTED_PARAMETER,
        regex: regex.parameter(paramTypes.quoted, regex.stringPattern(cfg.identTypes)),
        key: function key(v) {
          return function (_ref) {
            var tokenKey = _ref.tokenKey,
                quoteChar = _ref.quoteChar;
            return tokenKey.replace(new RegExp(escapeRegExp('\\' + quoteChar), 'gu'), quoteChar);
          }({
            tokenKey: v.slice(2, -1),
            quoteChar: v.slice(-1)
          });
        }
      }, {
        type: TokenType.NUMBERED_PARAMETER,
        regex: regex.parameter(paramTypes.numbered, '[0-9]+'),
        key: function key(v) {
          return v.slice(1);
        }
      }, {
        type: TokenType.POSITIONAL_PARAMETER,
        regex: paramTypes.positional ? new RegExp("[?]", "y") : undefined
      }]);
    } // filters out rules for token types whose regex is undefined

  }, {
    key: "validRules",
    value: function validRules(rules) {
      return rules.filter(function (rule) {
        return Boolean(rule.regex);
      });
    }
  }]);

  return Tokenizer;
}();
/**
 * Converts keywords (and keyword sequences) to canonical form:
 * - in uppercase
 * - single spaces between words
 */


export { Tokenizer as default };

var toCanonical = function toCanonical(v) {
  return equalizeWhitespace(v.toUpperCase());
};
//# sourceMappingURL=Tokenizer.js.map